//*******************************************************************************
//** Copyright © 2004,2005,2006, Xilinx, Inc. 
//** This design is confidential and proprietary of Xilinx, Inc. All Rights Reserved.
//*******************************************************************************
//**   ____  ____ 
//**  /   /\/   / 
//** /___/  \  /   Vendor: Xilinx 
//** \   \   \/    Version: 1.0
//**  \   \        Filename: dpll.v 
//**  /   /        Date Last Modified: 6/22/2006 
//** /___/   /\    Date Created: 6/1/2005
//** \   \  /  \ 
//**  \___\/\___\ 
//** 
//**   
//*******************************************************************************
//**
//**  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//**              provided to you "as is." Xilinx and its licensors make and you
//**              receive no warranties or conditions, express, implied,
//**              statutory or otherwise, and Xilinx specifically disclaims any
//**              implied warranties of merchantability, noninfringement, or
//**              fitness for a particular purpose. Xilinx does not warrant that
//**              the functions contained in these designs will meet your
//**              requirements, or that the operation of these designs will be
//**              uninterrupted or error free, or that defects in the Designs
//**              will be corrected. Furthermore, Xilinx does not warrant or
//**              make any representations regarding use or the results of the
//**              use of the designs in terms of correctness, accuracy,
//**              reliability, or otherwise.
//**
//**              LIMITATION OF LIABILITY. In no event will Xilinx or its
//**              licensors be liable for any loss of data, lost profits, cost
//**              or procurement of substitute goods or services, or for any
//**              special, incidental, consequential, or indirect damages
//**              arising from the use or operation of the designs or
//**              accompanying documentation, however caused and on any theory
//**              of liability. This limitation will apply even if Xilinx
//**              has been advised of the possibility of such damage. This
//**              limitation shall apply notwithstanding the failure of the
//**              essential purpose of any limited remedies herein.
//**
//*******************************************************************************
// Patent Pending
//-----------------------------------------------------------------
//  This is the top level for the PLL design.  This is the block
//  to instantiate into any other design.
//-----------------------------------------------------------------
`timescale 1ns/100ps
`ifdef TPDA
`else
 `define TPDB #0.1
 `define TPDA #0.1
`endif

module dpll (/*AUTOARG*/
   // Outputs
   SCLK, SDO, SYNC, vio_in, outoflock, sclki, 
   // Inputs
   refclki, vcoclk, vio_out, rstlock
   );
   input refclki;          // Reference Clock input
   input vcoclk;           // Clock from VCO
   output SCLK,SDO,SYNC;   // Output signals to control DAC
   input [63:0] vio_out;   // Chipscope vio control bus
   output [63:0] vio_in;   // Chipscope vio monitor bus
   output 	 outoflock;// signal to indicate out of lock condition 
   input 	 rstlock;  // Clear latched outoflock signal
   output 	 sclki;    // low-freq sclk output used for chipscope clock
   
   wire [3:0] bitsel;
   wire       clkendac,isync;
   wire [15:0] vc,vcx;
   wire [8:0]  phase_error;
   wire [31:0] integrator;
   wire       refclk,vcoclk;
   wire       rst;
   wire [8:0] ph_error;
   wire [8:0] f_error;
   wire [7:0] divcnt;
   wire [15:0] fixed_vc;
   wire        vc_sel;
   wire [3:0]  beta, alpha;
   wire [2:0]  refsel,vsel;
   reg 	      outoflock;
     
`ifdef SIM
   // This sections indicate default values for simultion
   // if implementing in a design which does not need programmablity
   // user can use this block to set values for synthesis
   assign      rst          = 0;    // reset signal for entire pll
   assign      divcnt[7:0]  = 8'h08;// controls speed of DAC clock
   assign      beta[3:0]    = 4'h0; // 1st order gain constant
   assign      alpha[3:0]   = 4'h4; // 2nd order gain constant
   assign      select       = 0;    // 0 phase detector : 1 = freq detector
   assign     fixed_vc[15:0] = 16'hbeef; // fixed DAC value when vc_sel = 0
   assign     vc_sel    = 1;        // select between fixed DAC value and dynamic
   assign     refsel       = 1;     // counter selection for freq detector
   assign     vsel       = 0;       // counter selection for freq detector
`else
   // remove this and chipscope busses if implementing in a design
   // which does not need programmability.
   assign     divcnt[7:0]  = vio_out[63:56];
   assign     rst          = vio_out[42];
   assign     refsel       = vio_out[2:0];
   assign     vsel       = vio_out[6:4];
   assign     beta[3:0]    = vio_out[55:52];
   assign     alpha[3:0]   = vio_out[51:48];
   assign     select       = vio_out[43];
   assign     fixed_vc[15:0] = vio_out[23:8];
   assign     vc_sel    = vio_out[41];
`endif


   
   BUFG refclk_bufg (.O(refclk) , .I(refclki));
   
  // PLL divider circuit controls DAC and LP clocking
   vcodiv vcodiv(
		 // Outputs
		 .clkendac		(clkendac),
		 .rstcnt		(rstcnt),
		 .bitsel		(bitsel[3:0]),
		 .sync			(isync),
		 .sclk			(sclki),
		 // Inputs
		 .clk			(vcoclk),
		 .divcnt		(divcnt)
		 );

   // Accumulating Bang-Bang phase detector
   pd pd(
	   // Outputs
	   .data			(),
	   .phase_error			(phase_error),
	   // Inputs
	   .refsig			(refclki),
	   .rstcnt			(rstcnt),
	   .vcoclk			(vcoclk),
	   .reset			(rst)
	   );
   
   // Frequency Detector
   fd fd_i(   // Outputs
	    .error	(f_error),
	    // Inputs
	    .refsig  (refclk), 
	    .vcoclk  (vcoclk),
	    .reset   (rst),
	    .refsel  (refsel),
	    .rstcnt  (rstcnt),
            .vcosel  (vsel)
	    );

   
   // mux input to loopfilter between ABB-Phase detector and Frequency Detector
   assign      ph_error = (select) ? f_error : phase_error;
   
   // digital dual path loop filter
   lp lp(
	 // Outputs
	 .vc				(vc[15:0]),
	 // Inputs
	 .clk				(vcoclk),
	 .clken				(clkendac),
	 .error	       		        (ph_error),
	 .beta				(beta),
	 .alpha				(alpha),
	 .rstcnt			(rstcnt),
	 .rstint                        (rst)
	 );

   // select between fixed and closed loop control voltage values
   assign vcx = (vc_sel) ? vc : fixed_vc;

   // out of lock register
   always@(posedge vcoclk or posedge rst)
      if(rst)
	 outoflock <= `TPDB 0;
      else if(rstlock)
	 outoflock <= `TPDB 0;
      else if (vc[11:0] > 12'hf00 | vc[11:0] < 12'h0ff)
	 outoflock <= `TPDB 1;
   
		 
   // Serial Peripheral Interface to DAC
   spi spi(
	   // Outputs
	   .SYNC			(SYNC),
	   .SDO				(SDO),
	   .SCLK			(SCLK),
	   // Inputs
	   .clk				(vcoclk),
	   .sclki			(sclki),
	   .data			(vcx[15:0]),
	   .isync			(isync),
	   .clkendac			(clkendac),
	   .bitsel			(bitsel[3:0])
	   );
   
   // assigns status signals to Chipscope Pro VIO bus
   assign  vio_in = {7'h00,ph_error,vcx,16'h0000};
   
endmodule // dpll
